from dislord.client import ApplicationClient

try:
    from flask import Flask, request
    app = Flask(__name__)
except ImportError:
    Flask = None
    request = None

    class FakeFlask:
        @staticmethod
        def route(self, *args, **kwargs): # noqa
            def decorator(func):
                return func
            return decorator

        def run(self, *args, **kwargs):
            raise RuntimeError("flask library needed in order to use server")

    app = FakeFlask()

__application_client: ApplicationClient


@app.route("/", methods=["POST"])
async def interactions_endpoint():
    raw_request = request.json
    signature = request.headers.get('X-Signature-Ed25519')
    timestamp = request.headers.get('X-Signature-Timestamp')
    print(f"👉 Request: {raw_request}")
    response = __application_client.interact(raw_request, signature, timestamp).as_server_response()
    print(f"🫴 Response: {response}")
    return response


def start_server(application_client, **kwargs):
    global __application_client
    __application_client = application_client
    app.run(**kwargs)






import dataclasses
import time
from dataclasses import dataclass
import json
import inspect
from enum import Enum, EnumType
from typing import get_type_hints, Union, Type

from models.type import PartialOptional


def cast(obj, type_hint, param_name=None, client=None):
    if type_hint is None:
        return obj

    if obj is None or obj is dataclasses.MISSING:
        if getattr(type_hint, "__origin__", None) is Union and type(None) in type_hint.__args__:
            return obj
        else:
            raise RuntimeError(f"Required field {param_name if param_name else ''} is not given for type: {type_hint}")

    if isinstance(obj, list) and getattr(type_hint, "__origin__", None) is list:
        return [cast(o, type_hint.__args__[0], param_name, client) for o in obj]
    elif isinstance(obj, dict) and getattr(type_hint, "__origin__", None) is dict:
        # return {ok: ov for (ok, ov) in obj.items()}
        raise NotImplementedError("TODO")
    elif getattr(type_hint, "__origin__", None) is obj.__class__:
        return obj

    if dataclasses.is_dataclass(type_hint):
        if type_hint.is_base_model():
            return type_hint.from_dict(obj, client)
        else:
            return type_hint.from_dict(obj)
    elif isinstance(type_hint, EnumType):
        return type_hint(obj)
    elif getattr(type_hint, '__origin__', None) is Union:
        for u_type in type_hint.__args__:
            return cast(obj, u_type, param_name, client)
    else:
        return obj


@dataclass
class BaseModel:
    @classmethod
    def from_dict(cls, env, client):
        type_hints = get_type_hints(cls)
        if cls == type(env):
            return env

        params = {}
        for p, hint in type_hints.items():
            params[p] = cast(env.get(p, dataclasses.MISSING), hint, p)
        obj = cls(**params)  # noqa
        obj.client = client
        return obj

    @classmethod
    def from_kwargs(cls, *, client, **kwargs):
        return cls.from_dict(kwargs, client)

    @staticmethod
    def is_base_model():
        return True


class EnhancedJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if dataclasses.is_dataclass(o):
            return dataclasses.asdict(o)
        if isinstance(o, Enum):
            return o.value
        if dataclasses.MISSING:
            return None
        return super().default(o)



from dataclasses import dataclass
from typing import Optional, Union
from enum import Enum

from dislord.models.base import BaseModel
from dislord.models.type import Snowflake
from models.command import ApplicationCommandOptionType
from models.user import User


class InteractionType(Enum):
    PING = 1
    APPLICATION_COMMAND = 2
    MESSAGE_COMPONENT = 3
    APPLICATION_COMMAND_AUTOCOMPLETE = 4
    MODAL_SUBMIT = 5


@dataclass
class ApplicationCommandInteractionDataOption(BaseModel):
    name: str
    type: ApplicationCommandOptionType
    value: Optional[Union[str, int, float, bool]]
    options: Optional['ApplicationCommandInteractionDataOption']
    focused: Optional[bool]


@dataclass
class InteractionData(BaseModel):
    id: Snowflake
    name: str
    type: int
    # resolved: Optional[ResolvedData] FIXME
    options: Optional[list[ApplicationCommandInteractionDataOption]]
    guild_id: Optional[Snowflake]
    target_id: Optional[Snowflake]


@dataclass
class Interaction(BaseModel):
    id: Snowflake
    application_id: Snowflake
    type: InteractionType
    token: str
    version: int
    # entitelements: list[Entitlement] FIXME

    data: Optional[InteractionData]
    guild_id: Optional[Snowflake]
    # channel: Optional[PartialChannel] FIXME
    channel_id: Optional[Snowflake]
    # member: Optional[GuildMember] FIXME
    user: Optional[User]
    # message: Optional[Message] FIXME
    app_permissions:  Optional[str]
    locale:  Optional[str]  # This is available on all interaction types except PING
    guild_locale: Optional[str]


class InteractionCallbackType(Enum):
    PONG = 1
    CHANNEL_MESSAGE_WITH_SOURCE = 4
    DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE = 5
    DEFERRED_UPDATE_MESSAGE = 6
    UPDATE_MESSAGE = 7  # Only valid for component-based interactions
    APPLICATION_COMMAND_AUTOCOMPLETE_RESULT = 8
    MODAL = 9  # Not available for MODAL_SUBMIT and PING interactions.
    PREMIUM_REQUIRED = 10  # Not available for APPLICATION_COMMAND_AUTOCOMPLETE and PING interactions.


@dataclass
class InteractionCallbackData(BaseModel):
    flags: Optional[int]
    # components: list[Component] FIXME
    # attachments: list[PartialAttachment] FIXME

    tts: Optional[bool]
    content: Optional[str]
    # embeds: Optional[list[Embed]] FIXME
    # allowed_mentions: Optional[AllowedMentions] FIXME


@dataclass
class InteractionResponse(BaseModel):
    type: InteractionCallbackType
    data: Optional[InteractionCallbackData] = None

    @staticmethod
    def pong():
        return InteractionResponse(InteractionCallbackType.PONG)

    @staticmethod
    def message(**kwargs):
        cls = InteractionResponse(InteractionCallbackType.CHANNEL_MESSAGE_WITH_SOURCE,
                                  InteractionCallbackData.from_dict(kwargs, None))
        return cls



import json
from dataclasses import MISSING
from typing import Callable

from discord_interactions import verify_key

from api import DiscordApi
from models.application import Application
from models.channel import Channel
from models.command import ApplicationCommand, ApplicationCommandType, ApplicationCommandOption
from models.guild import Guild, PartialGuild
from models.type import Snowflake
from models.user import User
from .error import DiscordApiException
from models.api import HttpResponse, HttpUnauthorized, HttpOk
from .models.interaction import Interaction, InteractionResponse, InteractionType


class ApplicationClient:
    _public_key: str
    _api: DiscordApi
    _commands: dict[Snowflake, dict[str, ApplicationCommand]] = {}
    _command_callbacks: dict[str, Callable] = {}
    _application: Application = MISSING
    _guilds: list[Guild] = MISSING

    def __init__(self, public_key, bot_token):
        self._public_key = public_key
        self._api = DiscordApi(self, bot_token)

    def interact(self, raw_request, signature, timestamp) -> HttpResponse:
        if signature is None or timestamp is None or not verify_key(json.dumps(raw_request, separators=(',', ':'))
                                                                        .encode('utf-8'), signature, timestamp,
                                                                    self._public_key):
            return HttpUnauthorized('Bad request signature')
        interaction = Interaction.from_dict(raw_request, self)
        if interaction.type == InteractionType.PING:  # PING
            response_data = InteractionResponse.pong()  # PONG
        elif interaction.type == InteractionType.APPLICATION_COMMAND:
            data = interaction.data
            command_name = data.name
            kwargs = {}
            for option in data.options:
                kwargs[option.name] = option.value
            response_data = self._command_callbacks[command_name](interaction=interaction, **kwargs)

        else:
            raise DiscordApiException(DiscordApiException.UNKNOWN_INTERACTION_TYPE.format(interaction.type))

        return HttpOk(response_data, headers={"Content-Type": "application/json"})

    def add_command(self, command: ApplicationCommand, callback: Callable):
        if self._commands.get(command.guild_id) is None:
            self._commands[command.guild_id] = {}
        self._command_callbacks[command.name] = callback
        self._commands.get(command.guild_id)[command.name] = command

    def command(self, *, name, description, dm_permission=True, nsfw=False, guild_ids: list[Snowflake] = None,
                options: list[ApplicationCommandOption] = None):
        if guild_ids is None:
            guild_ids = ["ALL"]

        def decorator(func):
            for guild_id in guild_ids:
                if guild_id == "ALL":
                    guild_id = None
                self.add_command(ApplicationCommand.from_kwargs(name=name, description=description,
                                                                type=ApplicationCommandType.CHAT_INPUT,
                                                                dm_permission=dm_permission, nsfw=nsfw,
                                                                guild_id=guild_id, options=options, client=self), func)
            return func

        return decorator

    def serverless_handler(self, event, context):
        if event['httpMethod'] == "POST":
            print(f"🫱 Full Event: {event}")
            raw_request = json.loads(event["body"])
            print(f"👉 Request: {raw_request}")
            raw_headers = event["headers"]
            signature = raw_headers.get('x-signature-ed25519')
            timestamp = raw_headers.get('x-signature-timestamp')
            response = self.interact(raw_request, signature, timestamp).as_serverless_response()
            print(f"🫴 Response: {response}")
            return response

    @property
    def application(self):
        if self._application is MISSING:
            self._application = self.get_application()
        return self._application

    @property
    def guilds(self):
        if self._guilds is MISSING:
            self._guilds = self._get_guilds()
        return self._guilds

    def get_application(self):
        return self._api.get("/applications/@me", type_hint=Application)

    def sync_commands(self, guild_id: Snowflake = None, guild_ids: list[Snowflake] = None,
                      application_id: Snowflake = None):
        if guild_ids:
            for g_id in guild_ids:
                self.sync_commands(guild_id=g_id, application_id=application_id)
        elif guild_id is None:
            return

        registered_commands = self._get_commands(guild_id)
        client_commands = self._commands.get(guild_id)
        missing_commands = list(client_commands.values()) if client_commands else []
        for registered_command in registered_commands:
            if registered_command not in missing_commands:
                self._delete_commands(command_id=registered_command.id, guild_id=guild_id,
                                      application_id=registered_command.application_id)
            else:
                missing_commands.remove(registered_command)

        for missing_command in missing_commands:
            self._register_command(missing_command, guild_id=guild_id, application_id=application_id)

    def _get_commands(self, guild_id: Snowflake = None, application_id: Snowflake = None,
                      with_localizations: bool = None) -> list[ApplicationCommand]:
        endpoint = f"/applications/{application_id if application_id else self.application.id}"
        if guild_id:
            endpoint += f"/guilds/{guild_id}"

        params = {}
        if with_localizations is not None:
            params["with_localizations"] = with_localizations

        return self._api.get(f"{endpoint}/commands", params=params, type_hint=list[ApplicationCommand])

    def _delete_commands(self, command_id: Snowflake,
                         guild_id: Snowflake = None, application_id: Snowflake = None) -> None:
        endpoint = f"/applications/{application_id if application_id else self.application.id}"
        if guild_id:
            endpoint += f"/guilds/{guild_id}"

        self._api.delete(f"{endpoint}/commands/{command_id}")



    def _register_command(self, application_command: ApplicationCommand,
                         guild_id: Snowflake = None, application_id: Snowflake = None) -> ApplicationCommand:
        endpoint = f"/applications/{application_id if application_id else self.application.id}"
        if guild_id:
            endpoint += f"/guilds/{guild_id}"
        return self._api.post(f"{endpoint}/commands", application_command, type_hint=ApplicationCommand)

    def get_user(self, user_id=None) -> User:
        return self._api.get(f"/users/{user_id if user_id else '@me'}", type_hint=User)

    def get_guild(self, guild_id) -> Guild:
        return self._api.get(f"/guilds/{guild_id}", type_hint=Guild)

    def _get_guilds(self) -> list[PartialGuild]:
        return self._api.get("/users/@me/guilds", type_hint=list[PartialGuild])

    def get_channel(self, channel_id) -> list[Channel]:
        return self._api.get(f"/channels/{channel_id}", type_hint=list[Channel])



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from models.base import BaseModel
from models.type import Snowflake
from models.user import User


class OverwriteType(Enum):
    ROLE = 0
    MEMBER = 1


@dataclass
class Overwrite(BaseModel):
    id: Snowflake
    type: OverwriteType
    allow: str
    deny: str


class ChannelType(Enum):
    GUILD_TEXT = 0
    DM = 1
    GUILD_VOICE = 2
    GROUP_DM = 3
    GUILD_CATEGORY = 4
    GUILD_ANNOUNCEMENT = 5
    ANNOUNCEMENT_THREAD = 10
    PUBLIC_THREAD = 11
    PRIVATE_THREAD = 12
    GUILD_STAGE_VOICE = 13
    GUILD_DIRECTORY = 14
    GUILD_FORUM = 15
    GUILD_MEDIA = 16


@dataclass
class Channel(BaseModel):
    id: Snowflake
    type: ChannelType
    guild_id: Optional[Snowflake]
    position: Optional[int]
    permission_overwrites: Optional[list[Overwrite]]
    name: Optional[str]
    topic: Optional[str]
    nsfw: Optional[bool]
    last_message_id: Optional[Snowflake]
    bitrate: Optional[int]
    user_limit: Optional[int]
    rate_limit_per_user: Optional[int]
    recipients: Optional[list[User]]
    icon: Optional[str]
    owner_id: Optional[Snowflake]
    application_id: Optional[Snowflake]
    managed: Optional[bool]
    parent_id: Optional[Snowflake]
    last_pin_timestamp: Optional[str]  # ISO8601 timestamp
    rtc_region: Optional[str]
    video_quality_mode: Optional[int]
    message_count: Optional[int]
    member_count: Optional[int]
    # thread_metadata: Optional[ThreadMetadata] FIXME
    # member: Optional[ThreadMember] FIXME
    default_auto_archive_duration: Optional[int]
    permissions: Optional[str]
    flags: Optional[int]
    total_message_sent: Optional[int]
    # available_tags: Optional[list[Tag]] FIXME
    applied_tags: Optional[list[Snowflake]]
    # default_reaction_emoji: Optional[DefaultReaction] FIXME
    default_thread_rate_limit_per_user: Optional[int]
    default_sort_order: Optional[int]
    default_forum_layout: Optional[int]



from typing import Optional
from dataclasses import dataclass, MISSING

from models.base import BaseModel
from models.type import Snowflake, PartialOptional


@dataclass
class Guild(BaseModel):
    id: Snowflake
    name: str
    # features: list[GuildFeature] FIXME: Add Enum
    icon: Optional[str]
    owner: Optional[bool]
    permissions: Optional[str]
    approximate_member_count: Optional[int]
    approximate_presence_count: Optional[int]

    owner_id: PartialOptional[Snowflake]
    afk_timeout: PartialOptional[int]
    verification_level: PartialOptional[int]
    default_message_notifications: PartialOptional[int]
    explicit_content_filter: PartialOptional[int]
    # roles: PartialOptional[list[Role]] FIXME: Add Enum
    # emojis: PartialOptional[list[Emoji]] FIXME: Add Enum
    mfa_level: PartialOptional[int]
    system_channel_flags: PartialOptional[int]
    premium_tier: PartialOptional[int]
    public_updates_channel_id: PartialOptional[Snowflake]
    nsfw_level: PartialOptional[int]
    premium_progress_bar_enabled: PartialOptional[bool]
    safety_alerts_channel_id: Optional[Snowflake]
    icon_hash: Optional[str]
    splash: Optional[str]
    discovery_splash: Optional[str]
    region: Optional[str]
    afk_channel_id: Optional[Snowflake]
    widget_enabled: Optional[bool]
    widget_channel_id: Optional[Snowflake]
    application_id: Optional[Snowflake]
    system_channel_id: Optional[Snowflake]
    rules_channel_id: Optional[Snowflake]
    max_presences: Optional[int]
    max_members: Optional[int]
    vanity_url_code: Optional[str]
    description: Optional[str]
    banner: Optional[str]
    preferred_locale: Optional[str]
    max_video_channel_users: Optional[int]
    # welcome_screen: Optional[WelcomeScreen] FIXME
    # stickers: Optional[list[Sticker]] FIXME


@dataclass
class PartialGuild(Guild):
    def __getattribute__(self, item):
        attr = super().__getattribute__(item)
        if attr is MISSING:
            self.__class__ = Guild
            guild = self.client.get_guild(self.id)
            for a in self.__annotations__:
                setattr(self, a, getattr(guild, a))
            return self.__getattribute__(item)
        else:
            return attr



import os

import dislord
from dislord.models.interaction import Interaction, InteractionResponse

PUBLIC_KEY = os.environ.get("PUBLIC_KEY")
BOT_TOKEN = os.environ.get("BOT_TOKEN")
OWNER_GUILD_ID = os.environ.get("OWNER_GUILD_ID")

client = dislord.ApplicationClient(PUBLIC_KEY, BOT_TOKEN)


@client.command(name="hello", description="Hello World!")
def hello(interaction: Interaction):
    return InteractionResponse.message(content=f"Hello {interaction.user.username}")


@client.command(name="sync", description="Sync Command", guild_ids=[OWNER_GUILD_ID])
def hello(interaction: Interaction):
    client.sync_commands()
    client.sync_commands(guild_ids=[g.id for g in client.guilds])
    return InteractionResponse.message(content="Sync Complete")


def serverless_handler(event, context):  # Not needed if using server
    client.serverless_handler(event, context)


if __name__ == '__main__':  # Not needed if using serverless
    client.sync_commands()
    client.sync_commands(guild_ids=[g.id for g in client.guilds])
    dislord.server.start_server(client, host='0.0.0.0', debug=True, port=8123)



from dataclasses import dataclass, MISSING, field
from enum import Enum
from typing import Optional, Union

from models.base import BaseModel
from models.channel import Channel, ChannelType
from models.locale import Locale
from models.type import Snowflake, PartialOptional
from models.user import User


class ApplicationCommandOptionType(Enum):
    SUB_COMMAND = 1
    SUB_COMMAND_GROUP = 2
    STRING = 3
    INTEGER = 4
    BOOLEAN = 5
    USER = 6
    CHANNEL = 7
    ROLE = 8
    MENTIONABLE = 9
    NUMBER = 10
    ATTACHMENT = 11

    def from_python_type(self, type_hint):
        python_mapping = {str: self.STRING, int: self.INTEGER, bool: self.BOOLEAN,
                          User: self.USER, Channel: self.CHANNEL,
                          # Role: self.ROLE, Mentionable: self.MENTIONABLE, FIXME
                          float: self.NUMBER,
                          # Attachment: self.ATTACHMENT FIXME
                          }
        par = python_mapping.get(type_hint)
        if par is None:
            raise RuntimeError(f"Unexpected command param type: {type_hint}")


@dataclass
class ApplicationCommandOptionChoice(BaseModel):
    name: str
    value: [str, int, float]
    name_localizations: Optional[dict[Locale, str]]


@dataclass
class ApplicationCommandOption(BaseModel):
    type: ApplicationCommandOptionType
    name: str
    description: str

    name_localizations: Optional[dict[Locale, str]]
    description_localizations: Optional[dict[Locale, str]]
    required: Optional[bool]
    choices: Optional[list[ApplicationCommandOptionChoice]]
    options: Optional[list['ApplicationCommandOption']]
    channel_types: Optional[list[ChannelType]]
    min_value: Optional[Union[int, float]]
    max_value: Optional[Union[int, float]]
    min_length: Optional[int]
    max_length: Optional[int]
    autocomplete: Optional[bool]


# _ApplicationCommandOption = Union[ApplicationCommandOption]

class ApplicationCommandType(Enum):
    CHAT_INPUT = 1
    USER = 2
    MESSAGE = 3


@dataclass
class ApplicationCommand(BaseModel):
    id: PartialOptional[Snowflake]
    application_id: PartialOptional[Snowflake]
    guild_id: Optional[Snowflake]
    name: str
    description: Optional[str]
    version: PartialOptional[Snowflake]

    type: Optional[ApplicationCommandType]
    name_localizations: Optional[dict[Locale, str]]
    description_localizations: Optional[dict[Locale, str]]
    options: Optional[list[ApplicationCommandOption]]
    default_member_permissions: Optional[str]
    dm_permission: Optional[bool]
    default_permission: Optional[bool]
    nsfw: Optional[bool]

    def __eq__(self, other):
        eq_list = ['guild_id', 'name', 'description', 'type', 'name_localization', 'description_localizations',
                   'options', 'default_member_permissions', 'dm_permission', 'default_permission', 'nsfw']
        result = True
        for eq_attr in eq_list:
            self_attr = getattr(self, eq_attr, None)
            other_attr = getattr(other, eq_attr, None)
            result = result and (self_attr if self_attr is not MISSING else None) \
                == (other_attr if other_attr is not MISSING else None)
        return result

    def __post_init__(self):
        if self.guild_id is not None and self.guild_id is not MISSING:
            self.dm_permission = None



import os

import dislord
from dislord.models.interaction import Interaction, InteractionResponse
from models.command import ApplicationCommandOption, ApplicationCommandOptionType

PUBLIC_KEY = os.environ.get("PUBLIC_KEY")
BOT_TOKEN = os.environ.get("BOT_TOKEN")

client = dislord.ApplicationClient(PUBLIC_KEY, BOT_TOKEN)


@client.command(name="add", description="Add together numbers",
                options=[ApplicationCommandOption.from_kwargs(name="a", description="First integer to add",
                                                              type=ApplicationCommandOptionType.INTEGER, client=client),
                         ApplicationCommandOption.from_kwargs(name="b", description="Second integer to add",
                                                              type=ApplicationCommandOptionType.INTEGER, client=client)
                         ]
                )
def add(interaction: Interaction, a: int, b: int):
    return InteractionResponse.message(content=f"{a} + {b} = {a+b}")


def serverless_handler(event, context):  # Not needed if using server
    client.serverless_handler(event, context)


if __name__ == '__main__':  # Not needed if using serverless
    client.sync_commands()
    client.sync_commands(guild_ids=[g.id for g in client.guilds])
    dislord.server.start_server(client, host='0.0.0.0', debug=True, port=8123)



class StateConfigurationException(Exception):
    MISSING_PUBLIC_TOKEN = "Public token has not been configured. Please use dislord.configure(...)"


class DiscordApiException(Exception):
    UNKNOWN_INTERACTION_TYPE = "Unknown interaction type %s"



import os

import dislord
from dislord.models.interaction import Interaction, InteractionResponse

PUBLIC_KEY = os.environ.get("PUBLIC_KEY")
BOT_TOKEN = os.environ.get("BOT_TOKEN")

client = dislord.ApplicationClient(PUBLIC_KEY, BOT_TOKEN)


@client.command(name="simple", description="Simple Command", guild_ids=['590643624358969350', '748515497913483416'])
def simple(interaction: Interaction):
    guilds = client.guilds
    return InteractionResponse.message(content="Servers:\n"+"\n".join([f"{g.name} {g.afk_timeout}" for g in guilds]))


def serverless_handler(event, context):  # Not needed if using server
    client.serverless_handler(event, context)


if __name__ == '__main__':  # Not needed if using serverless
    client.sync_commands()
    client.sync_commands(guild_ids=[g.id for g in client.guilds])
    dislord.server.start_server(client, host='0.0.0.0', debug=True, port=8123)



from dataclasses import dataclass
from typing import Optional

from models.base import BaseModel
from models.guild import PartialGuild
from models.type import Snowflake
from models.user import User


@dataclass
class Application(BaseModel):
    id: Snowflake
    name: str
    icon: Optional[str]
    description: str
    rpc_origins: Optional[list[str]]
    bot_public: bool
    bot_require_code_grant: bool
    bot: Optional[User]
    terms_of_service_url: Optional[str]
    privacy_policy_url: Optional[str]
    owner: Optional[User]
    summary: str  # depreciated v11
    verify_key: str
    # team: Optional[Team] FIXME
    guild: Optional[PartialGuild]
    primary_sku_id: Optional[Snowflake]
    slug: Optional[str]
    cover_image: Optional[str]
    flags: Optional[int]
    approximate_guild_count: Optional[int]
    redirect_uris: Optional[list[str]]
    interactions_endpoint_url: Optional[str]
    role_connections_verification_url: Optional[str]
    tags: Optional[list[str]]
    # install_params: Optional[InstallParams] FIXME
    custom_install_url: Optional[str]



from dataclasses import dataclass
from typing import Optional

from models.base import BaseModel
from models.type import Snowflake


@dataclass
class User(BaseModel):
    id: Snowflake
    username: str
    discriminator: str
    global_name: Optional[str]
    avatar: Optional[str]
    bot: Optional[bool]
    system: Optional[bool]
    mfa_enabled: Optional[bool]
    banner: Optional[str]
    accent_color: Optional[int]
    locale: Optional[str]
    verified: Optional[bool]
    email: Optional[str]
    flags: Optional[int]
    premium_type: Optional[int]
    public_flags: Optional[int]
    avatar_decoration: Optional[str]



import json
import requests

from error import DiscordApiException
from models.base import cast, EnhancedJSONEncoder

DISCORD_API_VERSION = 10
DISCORD_URL = f"https://discord.com/api/v{DISCORD_API_VERSION}"


# WARNING: Average time to call and get response from API is 25ms, not great to call lots if you want quick processing


class DiscordApi:
    def __init__(self, client, bot_token):
        self.client = client
        self.bot_token = bot_token
        self.auth_header = {"Authorization": "Bot " + self.bot_token}

    def get(self, endpoint: str, params: dict = None, type_hint: type = None, **kwargs):
        print(f"📨 Sending to Discord API GET {endpoint}, {params}")
        response = requests.get(DISCORD_URL + endpoint, params, **kwargs, headers=self.auth_header)
        if response.ok:
            return cast(json.loads(response.content), type_hint, client=self.client)
        else:
            raise DiscordApiException(f"{response.status_code} {response.text} error when calling discord API "
                                      f"URL: GET {endpoint} Params: {params}")

    def delete(self, endpoint: str, **kwargs):
        print(f"📨 Sending to Discord API DELETE {endpoint}")
        response = requests.delete(DISCORD_URL + endpoint, **kwargs, headers=self.auth_header)
        if response.ok:
            return
        else:
            raise DiscordApiException(f"{response.status_code} {response.text} error when calling discord API "
                                      f"URL: DELETE {endpoint}")

    def post(self, endpoint: str, body: object = None, type_hint: type = None, **kwargs):
        body_json = json.dumps(body, cls=EnhancedJSONEncoder)
        print(f"📨 Sending to Discord API POST {endpoint}, {body_json}")
        headers = self.auth_header
        headers["Content-Type"] = "application/json"
        response = requests.post(DISCORD_URL + endpoint, data=body_json,
                                 **kwargs, headers=headers)
        if response.ok:
            return cast(json.loads(response.content), type_hint, client=self.client)
        else:
            raise DiscordApiException(f"{response.status_code} {response.text} error when calling discord API "
                                      f"URL: POST {endpoint} Body: {body_json}")



from enum import Enum


class Locale(Enum):
    INDONESIAN = 'id'
    DANISH = 'da'
    GERMAN = 'de'
    BRITISH_ENGLISH = 'en-GB'
    AMERICAN_ENGLISH = 'en-US'
    SPAIN_SPANISH = 'es-ES'
    FRENCH = 'fr'
    CROATIAN = 'hr'
    ITALIAN = 'it'
    LITHUANIAN = 'lt'
    HUNGARIAN = 'hu'
    DUTCH = 'nl'
    NORWEGIAN = 'no'
    POLISH = 'pl'
    BRAZIL_PORTUGUESE = 'pt-BR'
    ROMANIAN = 'ro'
    FINNISH = 'fi'
    SWEDISH = 'sv-SE'
    VIETNAMESE = 'vi'
    TURKISH = 'tr'
    CZECH = 'cs'
    GREEK = 'el'
    BULGARIAN = 'bg'
    RUSSIAN = 'ru'
    UKRAINIAN = 'uk'
    HINDI = 'hi'
    THAI = 'th'
    CHINESE = 'zh-CN'
    JAPANESE = 'ja'
    TAIWAN_CHINESE = 'zh-TW'
    KOREAN = 'ko'

    def __str__(self) -> str:
        return self.value